/*
  quantiles.hpp

  Copyright (C) 2016 Taiji Yamada <taiji@aihara.co.jp>
*/
#ifndef _QUANTILES_HPP_
#define _QUANTILES_HPP_
#include <cmath>
#if __cplusplus < 201103L
#if !defined(BOOST_UTILITY_HPP)
template <typename InputIterator>
inline InputIterator next(InputIterator it) { return ++it; }
template <typename InputIterator, typename Difference>
inline InputIterator next(InputIterator it, Difference n)
{
  std::advance(it, n);
  return it;
}
template <typename InputIterator>
inline InputIterator prior(InputIterator it) { return --it; }
template <typename InputIterator, typename Difference>
inline InputIterator prior(InputIterator it, Difference n)
{
  std::advance(it, -n);
  return it;
}
#else
using boost::next;
#endif
#else
using std::next;
#endif

/* zero-quantile */
template <typename InputIterator>
inline
typename std::iterator_traits<InputIterator>::value_type least_in_sorted_container(const InputIterator first, const InputIterator last)
{
  (void)last;
  const InputIterator it(first);
  return *it;
}

/* one-quantile */
template <typename InputIterator>
inline
typename std::iterator_traits<InputIterator>::value_type greatest_in_sorted_container(const InputIterator first, const InputIterator last)
{
  const InputIterator it(next(first, (std::distance(first, last) - 1)));
  return *it;
}

/* two-quantile */
template <typename InputIterator>
inline
typename std::iterator_traits<InputIterator>::value_type median_in_sorted_container(const InputIterator first, const InputIterator last)
{
  typename std::iterator_traits<InputIterator>::difference_type pn((std::distance(first, last) - 1));
  InputIterator it(next(first, pn/2));
  return (std::distance(first, last) % 2) ? *it : (*it + *(next(it, 1)))/2;
}

/* p-quantile */
template <typename InputIterator>
inline
typename std::iterator_traits<InputIterator>::value_type quantile_in_sorted_container(const InputIterator first, const InputIterator last, const typename std::iterator_traits<InputIterator>::value_type p)
{
  typename std::iterator_traits<InputIterator>::difference_type pn((std::distance(first, last) - 1)), i(floor(pn*p));
  InputIterator it(next(first, i));
  return p*(*it) + (1 - p)*(*(next(it, 1)));
}

#endif
