/*
  quantiles_online.hpp

  Copyright (C) 2016 Taiji Yamada <taiji@aihara.co.jp>
*/
#ifndef _QUANTILES_ONLINE_HPP_
#define _QUANTILES_ONLINE_HPP_
#include "statistics_online.hpp"
#include "quantiles.hpp"

template <typename T>
class moving_quantiles_online : public moving_statistics_online<T> {
public:
  moving_quantiles_online(const size_t size = 100) : moving_statistics_online<T>(size), sorted(false)
  {
    sorted_storage = new T[size];
  }
  virtual ~moving_quantiles_online()
  {
    delete [] sorted_storage;
  }
  void add(const T v)
  {
    moving_statistics_online<T>::add(v);
    sorted = false;
  }
  T get_median()
  {
    const size_t size(moving_statistics_online<T>::size);
    const size_t index(moving_statistics_online<T>::get_index());
    const bool initial(moving_statistics_online<T>::is_initial());
    if (!sorted)
      sort();
    return median_in_sorted_container(sorted_storage, sorted_storage + (initial ? index : size));
  }
  T get_quantile(const T p)
  {
    const size_t size(moving_statistics_online<T>::size);
    const size_t index(moving_statistics_online<T>::get_index());
    const bool initial(moving_statistics_online<T>::is_initial());
    if (!sorted)
      sort();
    return quantile_in_sorted_container(sorted_storage, sorted_storage + (initial ? index : size), p);
  }
  void reset() { moving_statistics_online<T>::reset(); sorted = false; }
private:
  void sort()
  {
    const size_t size(moving_statistics_online<T>::size);
    const size_t index(moving_statistics_online<T>::get_index());
    const bool initial(moving_statistics_online<T>::is_initial());
    std::copy(moving_arithmetic_average_online<T>::get_storage(), moving_arithmetic_average_online<T>::get_storage() + (initial ? index : size), sorted_storage);
    std::sort(sorted_storage, sorted_storage + (initial ? index : size));
    sorted = true;
  }
  bool sorted;
  T *sorted_storage;
};

#endif
